@@ -22,7 +22,6 @@
 #include <config.h>
 
 #include <sys/file.h>
-#include <sys/stat.h>
 #include <gnome.h>
 #include <glade/glade.h>
 #include <gconf/gconf-client.h>
@@ -100,17 +99,8 @@ checkbox_toggled_cb (GtkWidget *widget, gpointer data)
 			"/apps/acme/use_pcm", state, NULL);
 }
 
-static void
-exec_clicked (GtkWidget *widget, gpointer data)
-{
-	GtkWidget *hbox = (GtkWidget *)data;
-
-	if (g_spawn_command_line_async ("acme", NULL) == TRUE)
-		gtk_widget_set_sensitive (hbox, FALSE);
-}
-
 static gboolean
-verify_double (char *current_path, int keycode)
+verify_double (char *current_path, int keycode, int *equal)
 {
 	gboolean found = FALSE;
 	int i = 0;
@@ -129,6 +119,7 @@ verify_double (char *current_path, int keycode)
 				keys[i].key_config, NULL);
 		if (val == keycode)
 		{
+			*equal = i;
 			found = TRUE;
 			break;
 		}
@@ -146,7 +137,8 @@ keys_edited_callback (GtkCellRendererText *cell, const char *path_string,
 	GtkTreeModel *model = (GtkTreeModel *)data;
 	GtkTreePath *path = gtk_tree_path_new_from_string (path_string);
 	GtkTreeIter iter;
-	char *gconf_path, *value_str;
+	char *gconf_path;
+	const char *value_str;
 	int value;
 	gboolean verified = FALSE;
 	GdkPixbuf *pix;
@@ -155,10 +147,14 @@ keys_edited_callback (GtkCellRendererText *cell, const char *path_string,
 	gtk_tree_model_get (model, &iter, PATH_COL, &gconf_path, -1);
 
 	if (gconf_path == NULL)
+	{
+		gtk_tree_path_free (path);
 		return;
+	}
 
 	if (keyval == GDK_C || keyval == GDK_c)
 	{
+		g_free (gconf_path);
 		gtk_tree_path_free (path);
 		return;
 	}
@@ -167,22 +163,28 @@ keys_edited_callback (GtkCellRendererText *cell, const char *path_string,
 	{
 		verified = TRUE;
 		value = -1;
-		value_str = g_strdup (_("Disabled"));
+		value_str = _("Disabled");
 		pix = disabled;
 	} else {
 		verified = FALSE;
 		value = keycode;
-		value_str = g_strdup (_("Enabled"));
+		value_str = _("Enabled");
 		pix = enabled;
 	}
 
 	if (verified == FALSE)
 	{
-		if (verify_double (gconf_path, value) == TRUE)
+		int duplicate;
+
+		if (verify_double (gconf_path, value, &duplicate) == TRUE)
 		{
+			char *msg;
+
 			gtk_tree_path_free (path);
-			g_free (value_str);
-			acme_error (_("This key is already bound to an action.\nPlease select another key."));
+			g_free (gconf_path);
+			msg = g_strdup_printf (_("This key is already bound to action '%s'.\nPlease select another key."), _(keys[duplicate].description));
+			acme_error (msg);
+			g_free (msg);
 			return;
 		}
 	}
@@ -194,6 +196,7 @@ keys_edited_callback (GtkCellRendererText *cell, const char *path_string,
 	gconf_client_set_int (conf_client,
 			gconf_path, value, NULL);
 
+	g_free (gconf_path);
 	gtk_tree_path_free (path);
 }
 
@@ -249,10 +252,10 @@ init_content_helper (GtkListStore *store, const char *path,
 
 	if (keyval > 0)
 	{
-		keyval_str = g_strdup (_("Enabled"));
+		keyval_str = _("Enabled");
 		pix = enabled;
 	} else {
-		keyval_str = g_strdup (_("Disabled"));
+		keyval_str = _("Disabled");
 		pix = disabled;
 	}
 
@@ -279,6 +282,11 @@ init_content (GtkTreeView *treeview)
 		init_content_helper (store, keys[i].key_config,
 				_(keys[i].description), keycode);
 	}
+
+	/* Sort the list since keys[i].description is gettextized */
+	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (store),
+					      DESC_COL,
+					      GTK_SORT_ASCENDING);
 }
 
 static void
@@ -345,7 +353,7 @@ static GtkWidget
 *init_gui (void)
 {
 	GladeXML *xml;
-	GtkWidget *window, *treeview, *checkbox, *exec_box, *exec;
+	GtkWidget *window, *treeview, *checkbox;
 	GdkPixbuf *pixbuf;
 
 	xml = glade_xml_new (ACME_DATA "acme-properties.glade", NULL, NULL);
@@ -374,12 +382,8 @@ static GtkWidget
 	g_signal_connect (G_OBJECT (checkbox), "toggled",
 			G_CALLBACK (checkbox_toggled_cb), NULL);
 
-	exec_box = glade_xml_get_widget (xml, "hbox8");
-	gtk_widget_set_sensitive (exec_box, !is_running ());
-
-	exec = glade_xml_get_widget (xml, "button1");
-	g_signal_connect (GTK_BUTTON (exec), "clicked",
-			G_CALLBACK (exec_clicked), exec_box);
+	if (is_running () == FALSE)
+		g_spawn_command_line_async ("acme", NULL);
 
 	gtk_widget_show_all (window);
 
