@@ -731,6 +731,10 @@ dialog_show (Acme *acme)
 	int orig_x, orig_y, orig_w, orig_h, orig_d;
 	int screen_w, screen_h;
 	int x, y;
+	int pointer_x, pointer_y;
+	GdkScreen *pointer_screen;
+	GdkRectangle geometry;
+	int monitor;
 
 	gtk_window_set_screen (GTK_WINDOW (acme->dialog), acme->current_screen);
 	gtk_widget_realize (GTK_WIDGET (acme->dialog));
@@ -739,11 +743,29 @@ dialog_show (Acme *acme)
 				 &orig_x, &orig_y,
 				 &orig_w, &orig_h, &orig_d);
 
-	screen_w = gdk_screen_get_width (acme->current_screen);
-	screen_h = gdk_screen_get_height (acme->current_screen);
+	pointer_screen = NULL;
+	gdk_display_get_pointer (gdk_screen_get_display (acme->current_screen),
+				 &pointer_screen, &pointer_x,
+				 &pointer_y, NULL);
+	if (pointer_screen != acme->current_screen) {
+		/* The pointer isn't on the current screen, so just
+		 * assume the default monitor
+		 */
+		monitor = 0;
+	} else {
+		monitor =
+			gdk_screen_get_monitor_at_point (acme->current_screen,
+							 pointer_x, pointer_y);
+	}
+		
+	gdk_screen_get_monitor_geometry (acme->current_screen, monitor,
+					 &geometry);
 
-	x = (screen_w - orig_w) / 2;
-	y = screen_h / 2 + (screen_h / 2 - orig_h) / 2;
+	screen_w = geometry.width;
+	screen_h = geometry.height;
+
+	x = ((screen_w - orig_w) / 2) + geometry.x;
+	y = geometry.y + (screen_h / 2) + (screen_h / 2 - orig_h) / 2;
 
 	gdk_window_move (GTK_WIDGET (acme->dialog)->window, x, y);
 
